package java.util;

import java.lang.reflect.Array;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.misc.SharedSecrets;
import java.io.Serializable;

public class EnumMap<K extends Enum<K>, V> extends AbstractMap<K, V> implements Serializable, Cloneable
{
    private final Class<K> keyType;
    private transient K[] keyUniverse;
    private transient Object[] vals;
    private transient int size;
    private static final Object NULL;
    private static final Enum<?>[] ZERO_LENGTH_ENUM_ARRAY;
    private transient Set<Map.Entry<K, V>> entrySet;
    private static final long serialVersionUID = 458661240069192865L;
    
    private Object maskNull(Object o) {
        return (o == null) ? EnumMap.NULL : o;
    }
    
    private V unmaskNull(Object o) {
        return (o == EnumMap.NULL) ? null : o;
    }
    
    public EnumMap(Class<K> keyType) {
        this.size = 0;
        this.keyType = keyType;
        this.keyUniverse = getKeyUniverse(keyType);
        this.vals = new Object[this.keyUniverse.length];
    }
    
    public EnumMap(EnumMap<K, ? extends V> enumMap) {
        this.size = 0;
        this.keyType = enumMap.keyType;
        this.keyUniverse = enumMap.keyUniverse;
        this.vals = (Object[])enumMap.vals.clone();
        this.size = enumMap.size;
    }
    
    public EnumMap(Map<K, ? extends V> map) {
        this.size = 0;
        if ((map instanceof EnumMap)) {
            EnumMap<Object, Object> enumMap = (EnumMap)map;
            this.keyType = enumMap.keyType;
            this.keyUniverse = enumMap.keyUniverse;
            this.vals = (Object[])enumMap.vals.clone();
            this.size = enumMap.size;
        }
        else {
            if (map.isEmpty()) {
                throw new IllegalArgumentException("Specified map is empty");
            }
            this.keyType = ((K)map.keySet().iterator().next()).getDeclaringClass();
            this.keyUniverse = getKeyUniverse(this.keyType);
            this.vals = new Object[this.keyUniverse.length];
            this.putAll(map);
        }
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean containsValue(Object maskNull) {
        maskNull = this.maskNull(maskNull);
        Object[] vals = this.vals;
        int length = vals.length;
        int i = 0;
        while (i < length) {
            if (maskNull.equals(vals[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.isValidKey(o) && this.vals[((Enum)o).ordinal()] != null;
    }
    
    private boolean containsMapping(Object o, Object o2) {
        return this.isValidKey(o) && this.maskNull(o2).equals(this.vals[((Enum)o).ordinal()]);
    }
    
    @Override
    public V get(Object o) {
        return this.isValidKey(o) ? this.unmaskNull(this.vals[((Enum)o).ordinal()]) : null;
    }
    
    @Override
    public V put(K k, V v) {
        this.typeCheck(k);
        int ordinal = k.ordinal();
        Object o = this.vals[ordinal];
        this.vals[ordinal] = this.maskNull(v);
        if (o == null) {
            ++this.size;
        }
        return this.unmaskNull(o);
    }
    
    @Override
    public V remove(Object o) {
        if (!this.isValidKey(o)) {
            return null;
        }
        int ordinal = ((Enum)o).ordinal();
        Object o2 = this.vals[ordinal];
        this.vals[ordinal] = null;
        if (o2 != null) {
            --this.size;
        }
        return this.unmaskNull(o2);
    }
    
    private boolean removeMapping(Object o, Object o2) {
        if (!this.isValidKey(o)) {
            return false;
        }
        int ordinal = ((Enum)o).ordinal();
        if (this.maskNull(o2).equals(this.vals[ordinal])) {
            this.vals[ordinal] = null;
            --this.size;
            return true;
        }
        return false;
    }
    
    private boolean isValidKey(Object o) {
        if (o == null) {
            return false;
        }
        Class<Object> class1 = o.getClass();
        return class1 == this.keyType || class1.getSuperclass() == this.keyType;
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        if ((map instanceof EnumMap)) {
            EnumMap<Object, Object> enumMap = (EnumMap)map;
            if (enumMap.keyType != this.keyType) {
                if (enumMap.isEmpty()) {
                    return;
                }
                throw new ClassCastException(enumMap.keyType + " != " + this.keyType);
            }
            else {
                for (int i = 0; i < this.keyUniverse.length; ++i) {
                    Object o = enumMap.vals[i];
                    if (o != null) {
                        if (this.vals[i] == null) {
                            ++this.size;
                        }
                        this.vals[i] = o;
                    }
                }
            }
        }
        else {
            super.putAll(map);
        }
    }
    
    @Override
    public void clear() {
        Arrays.fill(this.vals, null);
        this.size = 0;
    }
    
    @Override
    public Set<K> keySet() {
        Set<Object> keySet = this.keySet;
        if (keySet == null) {
            keySet = new EnumMap.KeySet();
            this.keySet = keySet;
            return keySet;
        }
        return keySet;
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new EnumMap.Values();
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        Set<Object> entrySet = this.entrySet;
        if (entrySet != null) {
            return entrySet;
        }
        return this.entrySet = new EnumMap.EntrySet();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof EnumMap)) {
            return this.equals((EnumMap)o);
        }
        if (!(o instanceof Map)) {
            return false;
        }
        Map<Object, Object> map = (Map)o;
        if (this.size != map.size()) {
            return false;
        }
        for (int i = 0; i < this.keyUniverse.length; ++i) {
            if (null != this.vals[i]) {
                Enum<Object> enum1 = this.keyUniverse[i];
                Object unmaskNull = this.unmaskNull(this.vals[i]);
                if (null == unmaskNull) {
                    if (null != map.get(enum1) || !map.containsKey(enum1)) {
                        return false;
                    }
                }
                else if (!unmaskNull.equals(map.get(enum1))) {
                    return false;
                }
            }
        }
        return true;
    }
    
    private boolean equals(EnumMap<?, ?> enumMap) {
        if (enumMap.keyType != this.keyType) {
            return this.size == 0 && enumMap.size == 0;
        }
        int i = 0;
        while (i < this.keyUniverse.length) {
            Object obj = this.vals[i];
            Object o = enumMap.vals[i];
            if (o != obj && (o == null || !o.equals(obj))) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        for (int i = 0; i < this.keyUniverse.length; ++i) {
            if (null != this.vals[i]) {
                n += this.entryHashCode(i);
            }
        }
        return n;
    }
    
    private int entryHashCode(int n) {
        return this.keyUniverse[n].hashCode() ^ this.vals[n].hashCode();
    }
    
    public EnumMap<K, V> clone() {
        EnumMap<Object, Object> enumMap;
        try {
            enumMap = (EnumMap)super.clone();
        }
        catch (CloneNotSupportedException ex) {
            throw new AssertionError();
        }
        enumMap.vals = (Object[])enumMap.vals.clone();
        enumMap.entrySet = null;
        return enumMap;
    }
    
    private void typeCheck(K k) {
        Class<Object> class1 = k.getClass();
        if (class1 != this.keyType && class1.getSuperclass() != this.keyType) {
            throw new ClassCastException(class1 + " != " + this.keyType);
        }
        return;
    }
    
    private static <K extends Enum<K>> K[] getKeyUniverse(Class<K> clazz) {
        return SharedSecrets.getJavaLangAccess().getEnumConstantsShared(clazz);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        objectOutputStream.writeInt(this.size);
        for (int i = this.size, n = 0; i > 0; ++n) {
            if (null != this.vals[n]) {
                objectOutputStream.writeObject(this.keyUniverse[n]);
                objectOutputStream.writeObject(this.unmaskNull(this.vals[n]));
                --i;
            }
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.keyUniverse = getKeyUniverse(this.keyType);
        this.vals = new Object[this.keyUniverse.length];
        for (int int1 = objectInputStream.readInt(), i = 0; i < int1; ++i) {
            this.put((Enum)objectInputStream.readObject(), objectInputStream.readObject());
        }
    }
    
    static Object access$500(EnumMap enumMap, Object o) {
        return enumMap.maskNull(o);
    }
    
    static boolean access$900(EnumMap enumMap, Object o, Object o2) {
        return enumMap.containsMapping(o, o2);
    }
    
    static boolean access$1000(EnumMap enumMap, Object o, Object o2) {
        return enumMap.removeMapping(o, o2);
    }
    
    static Object access$1200(EnumMap enumMap, Object o) {
        return enumMap.unmaskNull(o);
    }
    
    static int access$1600(EnumMap enumMap, int n) {
        return enumMap.entryHashCode(n);
    }
    
    static {
        NULL = new Object() {
            @Override
            public int hashCode() {
                return 0;
            }
            
            @Override
            public String toString() {
                return "java.util.EnumMap.NULL";
            }
        };
        ZERO_LENGTH_ENUM_ARRAY = new Enum[0];
    }
    
    private class KeySet extends AbstractSet<K>
    {
        @Override
        public Iterator<K> iterator() {
            return new EnumMap.KeyIterator();
        }
        
        @Override
        public int size() {
            return EnumMap.this.size;
        }
        
        @Override
        public boolean contains(Object o) {
            return EnumMap.this.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            int access$200 = EnumMap.this.size;
            EnumMap.this.remove(o);
            return EnumMap.this.size != access$200;
        }
        
        @Override
        public void clear() {
            EnumMap.this.clear();
        }
    }
    
    private class Values extends AbstractCollection<V>
    {
        @Override
        public Iterator<V> iterator() {
            return new EnumMap.ValueIterator();
        }
        
        @Override
        public int size() {
            return EnumMap.this.size;
        }
        
        @Override
        public boolean contains(Object o) {
            return EnumMap.this.containsValue(o);
        }
        
        @Override
        public boolean remove(Object access$500) {
            access$500 = EnumMap.access$500(EnumMap.this, access$500);
            int i = 0;
            while (i < EnumMap.this.vals.length) {
                if (access$500.equals(EnumMap.this.vals[i])) {
                    EnumMap.this.vals[i] = null;
                    EnumMap.this.size--;
                    return true;
                }
                ++i;
            }
            return false;
        }
        
        @Override
        public void clear() {
            EnumMap.this.clear();
        }
    }
    
    private class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return new EnumMap.EntryIterator();
        }
        
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return EnumMap.access$900(EnumMap.this, entry.getKey(), entry.getValue());
        }
        
        @Override
        public boolean remove(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return EnumMap.access$1000(EnumMap.this, entry.getKey(), entry.getValue());
        }
        
        @Override
        public int size() {
            return EnumMap.this.size;
        }
        
        @Override
        public void clear() {
            EnumMap.this.clear();
        }
        
        @Override
        public Object[] toArray() {
            return this.fillEntryArray(new Object[EnumMap.this.size]);
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            int size = this.size();
            if (array.length < size) {
                array = (Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), size);
            }
            if (array.length > size) {
                array[size] = null;
            }
            return (Object[])this.fillEntryArray(array);
        }
        
        private Object[] fillEntryArray(Object[] array) {
            int n = 0;
            for (int i = 0; i < EnumMap.this.vals.length; ++i) {
                if (EnumMap.this.vals[i] != null) {
                    array[n++] = new AbstractMap.SimpleEntry(EnumMap.this.keyUniverse[i], EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[i]));
                }
            }
            return array;
        }
    }
    
    private abstract class EnumMapIterator<T> implements Iterator<T>
    {
        int index;
        int lastReturnedIndex;
        
        private EnumMapIterator() {
            this.index = 0;
            this.lastReturnedIndex = -1;
        }
        
        @Override
        public boolean hasNext() {
            while (this.index < EnumMap.this.vals.length && EnumMap.this.vals[this.index] == null) {
                ++this.index;
            }
            return this.index != EnumMap.this.vals.length;
        }
        
        @Override
        public void remove() {
            this.checkLastReturnedIndex();
            if (EnumMap.this.vals[this.lastReturnedIndex] != null) {
                EnumMap.this.vals[this.lastReturnedIndex] = null;
                EnumMap.this.size--;
            }
            this.lastReturnedIndex = -1;
        }
        
        private void checkLastReturnedIndex() {
            if (this.lastReturnedIndex < 0) {
                throw new IllegalStateException();
            }
            return;
        }
    }
    
    private class KeyIterator extends EnumMap.EnumMapIterator<K>
    {
        @Override
        public K next() {
            if (!this.hasNext()) {
                throw new NoSuchElementException();
            }
            this.lastReturnedIndex = this.index++;
            return EnumMap.this.keyUniverse[this.lastReturnedIndex];
        }
    }
    
    private class ValueIterator extends EnumMap.EnumMapIterator<V>
    {
        @Override
        public V next() {
            if (!this.hasNext()) {
                throw new NoSuchElementException();
            }
            this.lastReturnedIndex = this.index++;
            return EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[this.lastReturnedIndex]);
        }
    }
    
    private class EntryIterator extends EnumMap.EnumMapIterator<Map.Entry<K, V>>
    {
        private EnumMap.EntryIterator.Entry lastReturnedEntry;
        
        @Override
        public Map.Entry<K, V> next() {
            if (!this.hasNext()) {
                throw new NoSuchElementException();
            }
            return this.lastReturnedEntry = new EnumMap.EntryIterator.Entry(this.index++);
        }
        
        @Override
        public void remove() {
            this.lastReturnedIndex = ((null == this.lastReturnedEntry) ? -1 : this.lastReturnedEntry.index);
            super.remove();
            this.lastReturnedEntry.index = this.lastReturnedIndex;
            this.lastReturnedEntry = null;
        }
        
        private class Entry implements Map.Entry<K, V>
        {
            private int index;
            
            private Entry(int index) {
                this.index = index;
            }
            
            @Override
            public K getKey() {
                this.checkIndexForEntryUse();
                return EnumMap.this.keyUniverse[this.index];
            }
            
            @Override
            public V getValue() {
                this.checkIndexForEntryUse();
                return EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[this.index]);
            }
            
            @Override
            public V setValue(V v) {
                this.checkIndexForEntryUse();
                Object access$1200 = EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[this.index]);
                EnumMap.this.vals[this.index] = EnumMap.access$500(EnumMap.this, v);
                return access$1200;
            }
            
            @Override
            public boolean equals(Object o) {
                if (this.index < 0) {
                    return o == this;
                }
                if (!(o instanceof Map.Entry)) {
                    return false;
                }
                Map.Entry<Object, Object> entry = (Map.Entry)o;
                Object access$1200 = EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[this.index]);
                Object value = entry.getValue();
                return entry.getKey() == EnumMap.this.keyUniverse[this.index] && (access$1200 == value || (access$1200 != null && access$1200.equals(value)));
            }
            
            @Override
            public int hashCode() {
                if (this.index < 0) {
                    return super.hashCode();
                }
                return EnumMap.access$1600(EnumMap.this, this.index);
            }
            
            @Override
            public String toString() {
                if (this.index < 0) {
                    return super.toString();
                }
                return EnumMap.this.keyUniverse[this.index] + "=" + EnumMap.access$1200(EnumMap.this, EnumMap.this.vals[this.index]);
            }
            
            private void checkIndexForEntryUse() {
                if (this.index < 0) {
                    throw new IllegalStateException("Entry was removed");
                }
                return;
            }
        }
    }
}
